package net.thecodersbreakfast.wickettips.presentation.wicket.enums;

/**
 * This class is used to determine the I18N key of an enum. Different strategies
 * can be chosen, but only one is active a a given time. This architecture is
 * required to avoid serialization when used from inside Wicket components.
 * 
 * @author Olivier Croisier
 */
public abstract class EnumMessageKeyProvider {

	/** The current key generation strategy. */
	private static EnumMessageKeyProvider provider = new DefaultEnumResourceKeyProvider();

	/**
	 * Encapsulates the I18N key generation algorithm (strategy)
	 * 
	 * @param <T>
	 *            The enum type
	 * @param enumValue
	 *            The enum value
	 * @return a String defining the I18N key for the provided enum
	 */
	public abstract <T extends Enum<T>> String computeMessageKey(T enumValue);

	/**
	 * Static accessor to the I18N key generation algorithm
	 */
	public static <T extends Enum<T>> String getMessageKey(T enumValue) {
		return EnumMessageKeyProvider.provider.computeMessageKey(enumValue);
	}

	// ================================================================================
	// Properties
	// ================================================================================

	public static EnumMessageKeyProvider getDefault() {
		return EnumMessageKeyProvider.provider;
	}

	public static void setDefault(EnumMessageKeyProvider provider) {
		EnumMessageKeyProvider.provider = provider;
	}

}
